/**
 * Font rendering routines.
 *
 * Copyright (C) 2006  Morten Hustveit
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#include <SDL/SDL.h>

#include <errno.h>
#include <stdio.h>
#include <string.h>
#include <wchar.h>

#ifndef WIN32
#include <arpa/inet.h>
#else
#include <winsock.h>
#endif

#include "font_32.h"
#include "error.h"

SDL_Surface* get_image(const char* name);

struct glyph_info
{
  int character;
  int bitmapidx;
  int top;
  int xskip;
  int image_width;
  int image_height;
  float s1;
  float t1;
  float s2;
  float t2;
};

static struct glyph_info* glyphs;
static int glyph_count;
static SDL_Surface* fontimage;

extern SDL_Surface* screen;

void load_font()
{
  int i;

  glyphs = (struct glyph_info*) font_32;

  glyph_count = size_font_32 / sizeof(glyphs[0]);

  for(i = 0; i < glyph_count * sizeof(glyphs[0]) / sizeof(int); ++i)
  {
    ((int*) glyphs)[i] = ntohl(((int*) glyphs)[i]);
  }

  fontimage = get_image("font.png");
}

int has_char(int ch)
{
  int i;

  for(i = 0; i < glyph_count; ++i)
  {
    if(glyphs[i].character == ch)
      return 1;
  }

  return 0;
}

void print_string(int x, int y, const wchar_t* string)
{
  SDL_Rect src, dest;
  const wchar_t* c;
  int i;
  int total_width = 0;

  for(c = string; *c; ++c)
  {
    for(i = 0; i < glyph_count; ++i)
    {
      if(glyphs[i].character == *c)
      {
        total_width += glyphs[i].xskip;

        break;
      }
    }
  }

  dest.x = x - total_width / 2;

  for(c = string; *c; ++c)
  {
    for(i = 0; i < glyph_count; ++i)
    {
      if(glyphs[i].character == *c)
        break;
    }

    if(i == glyph_count)
      continue;

    dest.y = y - glyphs[i].top;
    dest.w = glyphs[i].image_width;
    dest.h = glyphs[i].image_height;

    src.x = (int) (glyphs[i].s1 * 512.0f + 0.5f);
    src.y = (int) (glyphs[i].t1 * 512.0f + 0.5f);
    src.w = dest.w;
    src.h = dest.h;

    SDL_BlitSurface(fontimage, &src, screen, &dest);

    dest.x += glyphs[i].xskip;
  }
}
